@@ -121,11 +121,8 @@ class FakeStatResult:
     This is needed as `os.stat_result` has no possibility to set
     nanosecond times directly.
     """
-    _stat_float_times: bool = True
-
     def __init__(self, is_windows: bool, user_id: int, group_id: int,
                  initial_time: Optional[float] = None):
-        self._use_float: Optional[bool] = None
         self.st_mode: int = 0
         self.st_ino: Optional[int] = None
         self.st_dev: int = 0
@@ -138,16 +135,6 @@ class FakeStatResult:
         self._st_mtime_ns: int = self._st_atime_ns
         self._st_ctime_ns: int = self._st_atime_ns
 
-    @property
-    def use_float(self) -> bool:
-        if self._use_float is None:
-            return self.stat_float_times()
-        return self._use_float
-
-    @use_float.setter
-    def use_float(self, val: bool) -> None:
-        self._use_float = val
-
     def __eq__(self, other: Any) -> bool:
         return (
                 isinstance(other, FakeStatResult) and
@@ -171,7 +158,6 @@ class FakeStatResult:
         behavior of the real os.stat_result.
         """
         stat_result = copy(self)
-        stat_result.use_float = self.use_float
         return stat_result
 
     def set_from_stat_result(self, stat_result: os.stat_result) -> None:
@@ -187,27 +173,10 @@ class FakeStatResult:
         self._st_mtime_ns = stat_result.st_mtime_ns
         self._st_ctime_ns = stat_result.st_ctime_ns
 
-    @classmethod
-    def stat_float_times(cls, newvalue: Optional[bool] = None) -> bool:
-        """Determine whether a file's time stamps are reported as floats
-        or ints.
-
-        Calling without arguments returns the current value.
-        The value is shared by all instances of FakeOsModule.
-
-        Args:
-            newvalue: If `True`, mtime, ctime, atime are reported as floats.
-                Otherwise, they are returned as ints (rounding down).
-        """
-        if newvalue is not None:
-            cls._stat_float_times = bool(newvalue)
-        return cls._stat_float_times
-
     @property
     def st_ctime(self) -> Union[int, float]:
         """Return the creation time in seconds."""
-        ctime = self._st_ctime_ns / 1e9
-        return ctime if self.use_float else int(ctime)
+        return self._st_ctime_ns / 1e9
 
     @st_ctime.setter
     def st_ctime(self, val: Union[int, float]) -> None:
@@ -217,8 +186,7 @@ class FakeStatResult:
     @property
     def st_atime(self) -> Union[int, float]:
         """Return the access time in seconds."""
-        atime = self._st_atime_ns / 1e9
-        return atime if self.use_float else int(atime)
+        return self._st_atime_ns / 1e9
 
     @st_atime.setter
     def st_atime(self, val: Union[int, float]) -> None:
@@ -228,8 +196,7 @@ class FakeStatResult:
     @property
     def st_mtime(self) -> Union[int, float]:
         """Return the modification time in seconds."""
-        mtime = self._st_mtime_ns / 1e9
-        return mtime if self.use_float else int(mtime)
+        return self._st_mtime_ns / 1e9
 
     @st_mtime.setter
     def st_mtime(self, val: Union[int, float]) -> None:
@@ -254,13 +221,13 @@ class FakeStatResult:
         mode = 0
         st_mode = self.st_mode
         if st_mode & stat.S_IFDIR:
-            mode |= stat.FILE_ATTRIBUTE_DIRECTORY
+            mode |= stat.FILE_ATTRIBUTE_DIRECTORY  # type:ignore[attr-defined]
         if st_mode & stat.S_IFREG:
-            mode |= stat.FILE_ATTRIBUTE_NORMAL
+            mode |= stat.FILE_ATTRIBUTE_NORMAL  # type:ignore[attr-defined]
         if st_mode & (stat.S_IFCHR | stat.S_IFBLK):
-            mode |= stat.FILE_ATTRIBUTE_DEVICE
+            mode |= stat.FILE_ATTRIBUTE_DEVICE  # type:ignore[attr-defined]
         if st_mode & stat.S_IFLNK:
-            mode |= stat.FILE_ATTRIBUTE_REPARSE_POINT
+            mode |= stat.FILE_ATTRIBUTE_REPARSE_POINT  # type:ignore
         return mode
 
     @property
