@@ -24,7 +24,7 @@ import tempfile
 import unittest
 
 from pyfakefs import fake_filesystem_unittest
-from pyfakefs.fake_filesystem import is_root
+from pyfakefs.fake_filesystem import is_root, set_uid, USER_ID
 from pyfakefs.tests.test_utils import RealFsTestMixin
 
 is_windows = sys.platform == 'win32'
@@ -36,7 +36,10 @@ class RealFsTestCase(fake_filesystem_unittest.TestCase, RealFsTestMixin):
         RealFsTestMixin.__init__(self)
 
     def setUp(self):
+        RealFsTestMixin.setUp(self)
         self.cwd = os.getcwd()
+        self.uid = USER_ID
+        set_uid(1000)
         if not self.use_real_fs():
             self.setUpPyfakefs()
             self.filesystem = self.fs
@@ -46,10 +49,8 @@ class RealFsTestCase(fake_filesystem_unittest.TestCase, RealFsTestMixin):
             self.fs.set_disk_usage(1000, self.base_path)
 
     def tearDown(self):
-        if self.use_real_fs():
-            self.os.chdir(os.path.dirname(self.base_path))
-            shutil.rmtree(self.base_path, ignore_errors=True)
-            os.chdir(self.cwd)
+        set_uid(self.uid)
+        RealFsTestMixin.tearDown(self)
 
     @property
     def is_windows_fs(self):
@@ -59,6 +60,22 @@ class RealFsTestCase(fake_filesystem_unittest.TestCase, RealFsTestMixin):
 
 
 class FakeShutilModuleTest(RealFsTestCase):
+    @unittest.skipIf(is_windows, 'Posix specific behavior')
+    def test_catch_permission_error(self):
+        root_path = self.make_path('rootpath')
+        self.create_dir(root_path)
+        dir1_path = self.os.path.join(root_path, 'dir1')
+        dir2_path = self.os.path.join(root_path, 'dir2')
+        self.create_dir(dir1_path)
+        self.os.chmod(dir1_path, 0o555)  # remove write permissions
+        self.create_dir(dir2_path)
+        old_file_path = self.os.path.join(dir2_path, 'f1.txt')
+        new_file_path = self.os.path.join(dir1_path, 'f1.txt')
+        self.create_file(old_file_path)
+
+        with self.assertRaises(PermissionError):
+            shutil.move(old_file_path, new_file_path)
+
     def test_rmtree(self):
         directory = self.make_path('xyzzy')
         dir_path = os.path.join(directory, 'subdir')
@@ -90,7 +107,8 @@ class FakeShutilModuleTest(RealFsTestCase):
         file_path = os.path.join(dir_path, 'baz')
         self.create_file(file_path)
         self.os.chmod(file_path, 0o444)
-        self.assertRaises(OSError, shutil.rmtree, dir_path)
+        with self.assertRaises(OSError):
+            shutil.rmtree(dir_path)
         self.assertTrue(os.path.exists(file_path))
         self.os.chmod(file_path, 0o666)
 
@@ -103,7 +121,8 @@ class FakeShutilModuleTest(RealFsTestCase):
         self.create_file(file_path)
         self.os.chmod(dir_path, 0o555)
         if not is_root():
-            self.assertRaises(OSError, shutil.rmtree, dir_path)
+            with self.assertRaises(OSError):
+                shutil.rmtree(dir_path)
             self.assertTrue(os.path.exists(file_path))
             self.os.chmod(dir_path, 0o777)
         else:
@@ -129,12 +148,14 @@ class FakeShutilModuleTest(RealFsTestCase):
         file_path = os.path.join(dir_path, 'baz')
         self.create_file(file_path)
         with open(file_path):
-            self.assertRaises(OSError, shutil.rmtree, dir_path)
+            with self.assertRaises(OSError):
+                shutil.rmtree(dir_path)
         self.assertTrue(os.path.exists(dir_path))
 
     def test_rmtree_non_existing_dir(self):
         directory = 'nonexisting'
-        self.assertRaises(OSError, shutil.rmtree, directory)
+        with self.assertRaises(OSError):
+            shutil.rmtree(directory)
         try:
             shutil.rmtree(directory, ignore_errors=True)
         except OSError:
@@ -261,10 +282,8 @@ class FakeShutilModuleTest(RealFsTestCase):
         self.create_file(src_file)
         self.assertTrue(os.path.exists(src_file))
         self.assertFalse(os.path.exists(dst_directory))
-        self.assertRaises(OSError,
-                          shutil.copytree,
-                          src_file,
-                          dst_directory)
+        with self.assertRaises(OSError):
+            shutil.copytree(src_file, dst_directory)
 
     def test_move_file_in_same_filesystem(self):
         self.skip_real_fs()
@@ -378,8 +397,8 @@ class FakeCopyFileTest(RealFsTestCase):
         contents = 'contents of file'
         self.create_file(src_file, contents=contents)
         self.assertTrue(os.path.exists(src_file))
-        self.assertRaises(shutil.Error,
-                          shutil.copyfile, src_file, dst_file)
+        with self.assertRaises(shutil.Error):
+            shutil.copyfile(src_file, dst_file)
 
     def test_raises_if_dest_is_a_symlink_to_src(self):
         self.skip_if_symlink_not_supported()
@@ -389,8 +408,8 @@ class FakeCopyFileTest(RealFsTestCase):
         self.create_file(src_file, contents=contents)
         self.create_symlink(dst_file, src_file)
         self.assertTrue(os.path.exists(src_file))
-        self.assertRaises(shutil.Error,
-                          shutil.copyfile, src_file, dst_file)
+        with self.assertRaises(shutil.Error):
+            shutil.copyfile(src_file, dst_file)
 
     def test_succeeds_if_dest_exists_and_is_writable(self):
         src_file = self.make_path('xyzzy')
@@ -422,7 +441,8 @@ class FakeCopyFileTest(RealFsTestCase):
             with self.open(dst_file) as f:
                 self.assertEqual('contents of source file', f.read())
         else:
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
+            with self.assertRaises(OSError):
+                shutil.copyfile(src_file, dst_file)
 
         os.chmod(dst_file, 0o666)
 
@@ -439,7 +459,8 @@ class FakeCopyFileTest(RealFsTestCase):
         self.assertTrue(os.path.exists(src_file))
         self.assertTrue(os.path.exists(dst_dir))
         if not is_root():
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
+            with self.assertRaises(OSError):
+                shutil.copyfile(src_file, dst_file)
         else:
             shutil.copyfile(src_file, dst_file)
             self.assertTrue(os.path.exists(dst_file))
@@ -449,7 +470,8 @@ class FakeCopyFileTest(RealFsTestCase):
         src_file = self.make_path('xyzzy')
         dst_file = self.make_path('xyzzy_copy')
         self.assertFalse(os.path.exists(src_file))
-        self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
+        with self.assertRaises(OSError):
+            shutil.copyfile(src_file, dst_file)
 
     @unittest.skipIf(is_windows, 'Posix specific behavior')
     def test_raises_if_src_not_readable(self):
@@ -461,7 +483,8 @@ class FakeCopyFileTest(RealFsTestCase):
         os.chmod(src_file, 0o000)
         self.assertTrue(os.path.exists(src_file))
         if not is_root():
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
+            with self.assertRaises(OSError):
+                shutil.copyfile(src_file, dst_file)
         else:
             shutil.copyfile(src_file, dst_file)
             self.assertTrue(os.path.exists(dst_file))
@@ -472,10 +495,8 @@ class FakeCopyFileTest(RealFsTestCase):
         dst_file = self.make_path('xyzzy_copy')
         self.create_dir(src_file)
         self.assertTrue(os.path.exists(src_file))
-        if self.is_windows_fs:
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
-        else:
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_file)
+        with self.assertRaises(OSError):
+            shutil.copyfile(src_file, dst_file)
 
     def test_raises_if_dest_is_a_directory(self):
         src_file = self.make_path('xyzzy')
@@ -485,10 +506,8 @@ class FakeCopyFileTest(RealFsTestCase):
         self.create_dir(dst_dir)
         self.assertTrue(os.path.exists(src_file))
         self.assertTrue(os.path.exists(dst_dir))
-        if self.is_windows_fs:
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_dir)
-        else:
-            self.assertRaises(OSError, shutil.copyfile, src_file, dst_dir)
+        with self.assertRaises(OSError):
+            shutil.copyfile(src_file, dst_dir)
 
     def test_moving_dir_into_dir(self):
         # regression test for #515
