@@ -949,6 +949,98 @@ class RealFileOpenTest(FakeFileOpenTest):
         return True
 
 
+class FakeFileOpenWithOpenerTest(FakeFileOpenTestBase):
+    def opener(self, path, flags):
+        return self.os.open(path, flags)
+
+    def test_use_opener_with_read(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='test')
+        with self.open(file_path, opener=self.opener) as f:
+            assert f.read() == 'test'
+            with self.assertRaises(OSError):
+                f.write('foo')
+
+    def test_use_opener_with_read_plus(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='test')
+        with self.open(file_path, 'r+', opener=self.opener) as f:
+            assert f.read() == 'test'
+            assert f.write('bar') == 3
+        with self.open(file_path) as f:
+            assert f.read() == 'testbar'
+
+    def test_use_opener_with_write(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='foo')
+        with self.open(file_path, 'w', opener=self.opener) as f:
+            with self.assertRaises(OSError):
+                f.read()
+            assert f.write('bar') == 3
+        with self.open(file_path) as f:
+            assert f.read() == 'bar'
+
+    def test_use_opener_with_write_plus(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='test')
+        with self.open(file_path, 'w+', opener=self.opener) as f:
+            assert f.read() == ''
+            assert f.write('bar') == 3
+        with self.open(file_path) as f:
+            assert f.read() == 'bar'
+
+    def test_use_opener_with_append(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='foo')
+        with self.open(file_path, 'a', opener=self.opener) as f:
+            assert f.write('bar') == 3
+            with self.assertRaises(OSError):
+                f.read()
+        with self.open(file_path) as f:
+            assert f.read() == 'foobar'
+
+    def test_use_opener_with_append_plus(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='foo')
+        with self.open(file_path, 'a+', opener=self.opener) as f:
+            assert f.read() == ''
+            assert f.write('bar') == 3
+        with self.open(file_path) as f:
+            assert f.read() == 'foobar'
+
+    def test_use_opener_with_exclusive_write(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='test')
+        with self.assertRaises(OSError):
+            self.open(file_path, 'x', opener=self.opener)
+
+        file_path = self.make_path('bar')
+        with self.open(file_path, 'x', opener=self.opener) as f:
+            assert f.write('bar') == 3
+            with self.assertRaises(OSError):
+                f.read()
+        with self.open(file_path) as f:
+            assert f.read() == 'bar'
+
+    def test_use_opener_with_exclusive_plus(self):
+        file_path = self.make_path('foo')
+        self.create_file(file_path, contents='test')
+        with self.assertRaises(OSError):
+            self.open(file_path, 'x+', opener=self.opener)
+
+        file_path = self.make_path('bar')
+        with self.open(file_path, 'x+', opener=self.opener) as f:
+            assert f.write('bar') == 3
+            assert f.read() == ''
+        with self.open(file_path) as f:
+            assert f.read() == 'bar'
+
+
+class RealFileOpenWithOpenerTest(FakeFileOpenWithOpenerTest):
+    def use_real_fs(self):
+        return True
+
+
 @unittest.skipIf(sys.version_info < (3, 8),
                  'open_code only present since Python 3.8')
 class FakeFilePatchedOpenCodeTest(FakeFileOpenTestBase):
@@ -1486,6 +1578,7 @@ class FakeFileOpenLineEndingTest(FakeFileOpenTestBase):
         with self.open(file_path, mode='r', newline='\r\n') as f:
             self.assertEqual(['1\r\n', '2\n3\r4'], f.readlines())
 
+    @unittest.skipIf(sys.version_info >= (3, 10), "U flag no longer supported")
     def test_read_with_ignored_universal_newlines_flag(self):
         file_path = self.make_path('some_file')
         file_contents = b'1\r\n2\n3\r4'
@@ -1497,6 +1590,14 @@ class FakeFileOpenLineEndingTest(FakeFileOpenTestBase):
         with self.open(file_path, mode='U', newline='\r') as f:
             self.assertEqual('1\r\n2\n3\r4', f.read())
 
+    @unittest.skipIf(sys.version_info < (3, 11), "U flag still supported")
+    def test_universal_newlines_flag_not_supported(self):
+        file_path = self.make_path('some_file')
+        file_contents = b'1\r\n2\n3\r4'
+        self.create_file(file_path, contents=file_contents)
+        with self.assertRaises(ValueError):
+            self.open(file_path, mode='U', newline='\r')
+
     def test_write_with_newline_arg(self):
         file_path = self.make_path('some_file')
         with self.open(file_path, 'w', newline='') as f:
