@@ -22,40 +22,43 @@ python docs.
 
 import errno
 import os
+import pathlib
 import stat
 import sys
 import unittest
+from collections import namedtuple
+from unittest import mock
 
-from pyfakefs.extra_packages import pathlib, pathlib2
+from pyfakefs import fake_pathlib, fake_filesystem, fake_filesystem_unittest
 from pyfakefs.fake_filesystem import is_root
-
-from pyfakefs import fake_pathlib, fake_filesystem
-from pyfakefs.tests.test_utils import RealFsTestCase
+from pyfakefs.helpers import IS_PYPY
+from pyfakefs.tests.test_utils import RealFsTestMixin
 
 is_windows = sys.platform == 'win32'
 
 
-def skip_if_pathlib_36_not_available():
-    if sys.version_info < (3, 6) and not pathlib2:
-        raise unittest.SkipTest('Changed behavior in Python 3.6')
+class RealPathlibTestCase(fake_filesystem_unittest.TestCase, RealFsTestMixin):
+    is_windows = sys.platform == 'win32'
 
-
-def skip_if_pathlib_36_is_available():
-    if sys.version_info >= (3, 6) or pathlib2:
-        raise unittest.SkipTest('Changed behavior in Python 3.6')
-
-
-class RealPathlibTestCase(RealFsTestCase):
     def __init__(self, methodName='runTest'):
-        super(RealPathlibTestCase, self).__init__(methodName)
-        self.pathlib = pathlib or pathlib2
-        self.path = None
+        fake_filesystem_unittest.TestCase.__init__(self, methodName)
+        RealFsTestMixin.__init__(self)
 
     def setUp(self):
-        super().setUp()
+        RealFsTestMixin.setUp(self)
+        self.filesystem = None
+        self.real_os = os
         if not self.use_real_fs():
-            self.pathlib = fake_pathlib.FakePathlibModule(self.filesystem)
-        self.path = self.pathlib.Path
+            self.setUpPyfakefs()
+            self.filesystem = self.fs
+            self.create_basepath()
+        self.pathlib = pathlib
+        self.path = pathlib.Path
+        self.os = os
+        self.open = open
+
+    def use_real_fs(self):
+        return False
 
 
 class FakePathlibInitializationTest(RealPathlibTestCase):
@@ -187,7 +190,8 @@ class FakePathlibInitializationWithDriveTest(RealPathlibTestCase):
 
 
 class RealPathlibInitializationWithDriveTest(
-        FakePathlibInitializationWithDriveTest):
+    FakePathlibInitializationWithDriveTest
+):
     def use_real_fs(self):
         return True
 
@@ -238,6 +242,13 @@ class FakePathlibPurePathTest(RealPathlibTestCase):
         with self.assertRaises(ValueError):
             self.path('passwd').relative_to('/usr')
 
+    @unittest.skipIf(sys.version_info < (3, 9),
+                     'is_relative_to new in Python 3.9')
+    def test_is_relative_to(self):
+        path = self.path('/etc/passwd')
+        self.assertTrue(path.is_relative_to('/etc'))
+        self.assertFalse(path.is_relative_to('/src'))
+
     def test_with_name(self):
         self.check_windows_only()
         self.assertEqual(
@@ -381,20 +392,39 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
         # we get stat.S_IFLNK | 0o755 under MacOs
         self.assertEqual(link_stat.st_mode, stat.S_IFLNK | 0o777)
 
-    @unittest.skipIf(sys.platform == 'darwin',
-                     'Different behavior under MacOs')
     def test_lchmod(self):
         self.skip_if_symlink_not_supported()
         file_stat = self.os.stat(self.file_path)
         link_stat = self.os.lstat(self.file_link_path)
-        if not hasattr(os, "lchmod"):
+        if not hasattr(self.real_os, "lchmod"):
             with self.assertRaises(NotImplementedError):
                 self.path(self.file_link_path).lchmod(0o444)
         else:
             self.path(self.file_link_path).lchmod(0o444)
             self.assertEqual(file_stat.st_mode, stat.S_IFREG | 0o666)
-            # we get stat.S_IFLNK | 0o755 under MacOs
-            self.assertEqual(link_stat.st_mode, stat.S_IFLNK | 0o444)
+            # the exact mode depends on OS and Python version
+            self.assertEqual(link_stat.st_mode & 0o777700,
+                             stat.S_IFLNK | 0o700)
+
+    @unittest.skipIf(
+        sys.version_info < (3, 10),
+        "follow_symlinks argument new in Python 3.10"
+    )
+    def test_chmod_no_followsymlinks(self):
+        self.skip_if_symlink_not_supported()
+        file_stat = self.os.stat(self.file_path)
+        link_stat = self.os.lstat(self.file_link_path)
+        if (self.real_os.chmod not in os.supports_follow_symlinks
+                or IS_PYPY):
+            with self.assertRaises(NotImplementedError):
+                self.path(self.file_link_path).chmod(0o444,
+                                                     follow_symlinks=False)
+        else:
+            self.path(self.file_link_path).chmod(0o444, follow_symlinks=False)
+            self.assertEqual(file_stat.st_mode, stat.S_IFREG | 0o666)
+            # the exact mode depends on OS and Python version
+            self.assertEqual(link_stat.st_mode & 0o777700,
+                             stat.S_IFLNK | 0o700)
 
     def test_resolve(self):
         self.create_dir(self.make_path('antoine', 'docs'))
@@ -411,11 +441,6 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
                 self.os.path.realpath(
                     self.make_path('antoine', 'setup.py'))))
 
-    def test_resolve_nonexisting_file(self):
-        skip_if_pathlib_36_is_available()
-        path = self.path('/foo/bar')
-        self.assert_raises_os_error(errno.ENOENT, path.resolve)
-
     def test_stat_file_in_unreadable_dir(self):
         self.check_posix_only()
         dir_path = self.make_path('some_dir')
@@ -434,32 +459,14 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
         file_path = self.os.path.join(dir_path, 'some_file')
         self.create_file(file_path)
         self.os.chmod(dir_path, 0o000)
-        iter = self.path(dir_path).iterdir()
+        it = self.path(dir_path).iterdir()
         if not is_root():
-            self.assert_raises_os_error(errno.EACCES, list, iter)
+            self.assert_raises_os_error(errno.EACCES, list, it)
         else:
-            path = str(list(iter)[0])
+            path = str(list(it)[0])
             self.assertTrue(path.endswith('some_file'))
 
-    @unittest.skipIf(not is_windows, 'Windows specific behavior')
-    def test_resolve_file_as_parent_windows(self):
-        skip_if_pathlib_36_is_available()
-        self.check_windows_only()
-        self.create_file(self.make_path('a_file'))
-        path = self.path(self.make_path('a_file', 'this can not exist'))
-        self.assert_raises_os_error(errno.ENOENT, path.resolve)
-
-    @unittest.skipIf(is_windows, 'POSIX specific behavior')
-    def test_resolve_file_as_parent_posix(self):
-        skip_if_pathlib_36_is_available()
-        self.check_posix_only()
-        self.create_file(self.make_path('a_file'))
-        path = self.path(
-            self.make_path('', 'a_file', 'this can not exist'))
-        self.assert_raises_os_error(errno.ENOTDIR, path.resolve)
-
-    def test_resolve_nonexisting_file_after_36(self):
-        skip_if_pathlib_36_not_available()
+    def test_resolve_nonexisting_file(self):
         path = self.path(
             self.make_path('/path', 'to', 'file', 'this can not exist'))
         self.assertEqual(path, path.resolve())
@@ -550,6 +557,19 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.assertTrue(self.os.path.exists(path_name))
         self.check_contents(path_name, 'ανοησίες'.encode('greek'))
 
+    @unittest.skipIf(sys.version_info < (3, 10),
+                     "newline argument new in Python 3.10")
+    def test_write_with_newline_arg(self):
+        path = self.path(self.make_path('some_file'))
+        path.write_text('1\r\n2\n3\r4', newline='')
+        self.check_contents(path, b'1\r\n2\n3\r4')
+        path.write_text('1\r\n2\n3\r4', newline='\n')
+        self.check_contents(path, b'1\r\n2\n3\r4')
+        path.write_text('1\r\n2\n3\r4', newline='\r\n')
+        self.check_contents(path, b'1\r\r\n2\r\n3\r4')
+        path.write_text('1\r\n2\n3\r4', newline='\r')
+        self.check_contents(path, b'1\r\r2\r3\r4')
+
     def test_read_bytes(self):
         path_name = self.make_path('binary_file')
         self.create_file(path_name, contents=b'Binary file contents')
@@ -635,12 +655,46 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         path = self.path(link_name)
         path.symlink_to(file_name)
         self.assertTrue(self.os.path.exists(link_name))
-        # file_obj = self.filesystem.ResolveObject(file_name)
-        # linked_file_obj = self.filesystem.ResolveObject(link_name)
-        # self.assertEqual(file_obj, linked_file_obj)
-        # link__obj = self.filesystem.LResolveObject(link_name)
         self.assertTrue(path.is_symlink())
 
+    @unittest.skipIf(sys.version_info < (3, 8),
+                     'link_to new in Python 3.8')
+    def test_link_to(self):
+        self.skip_if_symlink_not_supported()
+        file_name = self.make_path('foo', 'bar.txt')
+        self.create_file(file_name)
+        self.assertEqual(1, self.os.stat(file_name).st_nlink)
+        link_name = self.make_path('link_to_bar')
+        path = self.path(file_name)
+        path.link_to(link_name)
+        self.assertTrue(self.os.path.exists(link_name))
+        self.assertFalse(path.is_symlink())
+        self.assertEqual(2, self.os.stat(file_name).st_nlink)
+
+    @unittest.skipIf(sys.version_info < (3, 10),
+                     'hardlink_to new in Python 3.10')
+    def test_hardlink_to(self):
+        self.skip_if_symlink_not_supported()
+        file_name = self.make_path('foo', 'bar.txt')
+        self.create_file(file_name)
+        self.assertEqual(1, self.os.stat(file_name).st_nlink)
+        link_path = self.path(self.make_path('link_to_bar'))
+        path = self.path(file_name)
+        link_path.hardlink_to(path)
+        self.assertTrue(self.os.path.exists(link_path))
+        self.assertFalse(path.is_symlink())
+        self.assertEqual(2, self.os.stat(file_name).st_nlink)
+
+    @unittest.skipIf(sys.version_info < (3, 9),
+                     'readlink new in Python 3.9')
+    def test_readlink(self):
+        self.skip_if_symlink_not_supported()
+        link_path = self.make_path('foo', 'bar', 'baz')
+        target = self.make_path('tarJAY')
+        self.create_symlink(link_path, target)
+        path = self.path(link_path)
+        self.assert_equal_paths(path.readlink(), self.path(target))
+
     def test_mkdir(self):
         dir_name = self.make_path('foo', 'bar')
         self.assert_raises_os_error(errno.ENOENT,
@@ -839,6 +893,12 @@ class FakePathlibUsageInOsFunctionsTest(RealPathlibTestCase):
         self.assertEqual(self.os.path.isfile(path),
                          self.os.path.isfile(self.path(path)))
 
+    def test_isfile_not_readable(self):
+        path = self.make_path('foo', 'bar', 'baz')
+        self.create_file(path, perm=0)
+        self.assertEqual(self.os.path.isfile(path),
+                         self.os.path.isfile(self.path(path)))
+
     def test_islink(self):
         path = self.make_path('foo', 'bar', 'baz')
         self.create_file(path)
@@ -908,13 +968,25 @@ class FakePathlibUsageInOsFunctionsTest(RealPathlibTestCase):
         self.os.makedirs(self.path(path))
         self.assertTrue(self.os.path.exists(path))
 
-    @unittest.skipIf(is_windows, 'os.readlink seems not to support '
-                                 'path-like objects under Windows')
+    @unittest.skipIf(is_windows and sys.version_info < (3, 8),
+                     'os.readlink does not to support path-like objects '
+                     'under Windows before Python 3.8')
     def test_readlink(self):
+        self.skip_if_symlink_not_supported()
         link_path = self.make_path('foo', 'bar', 'baz')
         target = self.make_path('tarJAY')
         self.create_symlink(link_path, target)
-        self.assertEqual(self.os.readlink(self.path(link_path)), target)
+        self.assert_equal_paths(self.os.readlink(self.path(link_path)), target)
+
+    @unittest.skipIf(is_windows and sys.version_info < (3, 8),
+                     'os.readlink does not to support path-like objects '
+                     'under Windows before Python 3.8')
+    def test_readlink_bytes(self):
+        self.skip_if_symlink_not_supported()
+        link_path = self.make_path(b'foo', b'bar', b'baz')
+        target = self.make_path(b'tarJAY')
+        self.create_symlink(link_path, target)
+        self.assert_equal_paths(self.os.readlink(self.path(link_path)), target)
 
     def test_remove(self):
         path = self.make_path('test.txt')
@@ -968,7 +1040,22 @@ class FakePathlibUsageInOsFunctionsTest(RealPathlibTestCase):
     def test_stat(self):
         path = self.make_path('foo', 'bar', 'baz')
         self.create_file(path, contents='1234567')
-        self.assertEqual(self.os.stat(path), self.os.stat(self.path(path)))
+        self.assertEqual(self.os.stat(path), self.path(path).stat())
+
+    @unittest.skipIf(sys.version_info < (3, 10), "New in Python 3.10")
+    def test_stat_follow_symlinks(self):
+        self.check_posix_only()
+        directory = self.make_path('foo')
+        base_name = 'bar'
+        file_path = self.path(self.os.path.join(directory, base_name))
+        link_path = self.path(self.os.path.join(directory, 'link'))
+        contents = "contents"
+        self.create_file(file_path, contents=contents)
+        self.create_symlink(link_path, base_name)
+        self.assertEqual(len(contents),
+                         link_path.stat(follow_symlinks=True)[stat.ST_SIZE])
+        self.assertEqual(len(base_name),
+                         link_path.stat(follow_symlinks=False)[stat.ST_SIZE])
 
     def test_utime(self):
         path = self.make_path('some_file')
@@ -985,6 +1072,49 @@ class FakePathlibUsageInOsFunctionsTest(RealPathlibTestCase):
         st = self.os.stat(path)
         self.assertEqual(4, st.st_size)
 
+    @unittest.skipIf(sys.platform == 'win32',
+                     'no pwd and grp modules in Windows')
+    def test_owner_and_group_posix(self):
+        path = self.make_path('some_file')
+        self.create_file(path)
+        self.assertTrue(self.path(path).owner())
+        self.assertTrue(self.path(path).group())
+
+    @unittest.skipIf(sys.platform == 'win32',
+                     'no pwd and grp modules in Windows')
+    def test_changed_owner_and_group(self):
+        def fake_getpwuid(uid):
+            if uid == 42:
+                user_struct = namedtuple('user', 'pw_name, pw_uid')
+                user_struct.pw_name = 'NewUser'
+                return user_struct
+            raise KeyError
+
+        def fake_getgrgid(uid):
+            if uid == 5:
+                group_struct = namedtuple('group', 'gr_name, gr_gid')
+                group_struct.gr_name = 'NewGroup'
+                return group_struct
+            raise KeyError
+
+        self.skip_real_fs()
+        path = self.make_path('some_file')
+        self.create_file(path)
+        self.os.chown(path, 42, 5)
+        with mock.patch('pwd.getpwuid', fake_getpwuid):
+            with mock.patch('grp.getgrgid', fake_getgrgid):
+                self.assertEqual('NewUser', self.path(path).owner())
+                self.assertEqual('NewGroup', self.path(path).group())
+
+    def test_owner_and_group_windows(self):
+        self.check_windows_only()
+        path = self.make_path('some_file')
+        self.create_file(path)
+        with self.assertRaises(NotImplementedError):
+            self.path(path).owner()
+        with self.assertRaises(NotImplementedError):
+            self.path(path).group()
+
 
 class RealPathlibUsageInOsFunctionsTest(FakePathlibUsageInOsFunctionsTest):
     def use_real_fs(self):
@@ -1051,4 +1181,4 @@ class FakeFilesystemPathLikeObjectTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest.main(verbosity=2)
